unit mainform;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, Menus, ExtCtrls,
  StdCtrls, UnTerminal, TermVT;

type

  { TForm1 }

  TForm1 = class(TForm)
    BTStart: TButton;
    BTSend: TButton;
    BTStop: TButton;
    BTQuit: TButton;
    CBCommands: TComboBox;
    CBSendSelected: TCheckBox;
    EDEngine: TLabeledEdit;
    EDEngineDir: TLabeledEdit;
    EDCommand: TLabeledEdit;
    LBProcessState: TLabel;
    MIAbout: TMenuItem;
    MMEngineOut: TMemo;
    MIDemo: TMenuItem;
    MIQuit: TMenuItem;
    MMMenu: TMainMenu;
    Panel1: TPanel;
    Separator1: TMenuItem;
    procedure BTQuitClick(Sender: TObject);
    procedure BTSendClick(Sender: TObject);
    procedure BTStartClick(Sender: TObject);
    procedure BTStopClick(Sender: TObject);
    procedure CBCommandsChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure MIAboutClick(Sender: TObject);
  private
    procedure AddLine(HeightScr: integer);
    procedure ChangeState(State: string; APoint: TPoint);
    procedure InitScreen(const AGrid: TtsGrid; iFrom, iTo: integer);
    procedure RefreshLine(const AGrid: TtsGrid; iFrom, HeightScr: integer);
    procedure RefreshLines(const AGrid: TtsGrid; iFrom, iTo, HeightScr: integer);
  public

  end;

var
  Form1: TForm1;
  Proc1: TConsoleProc;

implementation

uses
  TypInfo, AboutForm;

{$R *.lfm}

procedure TForm1.BTStartClick(Sender: TObject);
begin
  if SetCurrentDir(EDEngineDir.Text) then
    Proc1.Open(EDEngine.Text, EmptyStr)
  else
    ShowMessage(Format('Cannot change directory to "%s"', [EDEngineDir.Text]));
end;

procedure TForm1.BTSendClick(Sender: TObject);
begin
  if Length(EDCommand.Text) > 0 then
    Proc1.SendLn(EDCommand.Text)
  else
    ShowMessage('No command provided');
end;

procedure TForm1.BTQuitClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.BTStopClick(Sender: TObject);
begin
  Proc1.Close;
end;

procedure TForm1.CBCommandsChange(Sender: TObject);
begin
  EDCommand.Text := CBCommands.Items[CBCommands.ItemIndex];
  if CBSendSelected.Checked then
    BTSend.Click;
end;

procedure TForm1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Proc1.Destroy;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Proc1 := TConsoleProc.Create(nil);
  Proc1.LineDelimSend  := LDS_CRLF;
  Proc1.OnInitScreen   := @InitScreen;
  Proc1.OnRefreshLine  := @RefreshLine;
  Proc1.OnRefreshLines := @RefreshLines;
  Proc1.OnAddLine      := @AddLine;
  Proc1.OnChangeState  := @ChangeState;

  CBCommands.Items.LoadFromFile(ExtractFilePath(ParamStr(0)) + 'commands.txt');

{$IFDEF CPU32}
  EDEngine.Text := EDEngine.Text + '_x32';
{$ELSE}
  EDEngine.Text := EDEngine.Text + '_x64';
{$ENDIF}
{$IFDEF WINDOWS}
  EDEngine.Text := EDEngine.Text + '.exe';
{$ENDIF}

  LBProcessState.Caption := Format('Process status: %s', [
    GetEnumName(TypeInfo(TEstadoCon), Ord(Proc1.State))
  ]);

  Color := clDefault;
end;

procedure TForm1.MIAboutClick(Sender: TObject);
begin
  Form2.ShowModal;
end;

procedure TForm1.AddLine(HeightScr: integer);
begin
  MMEngineOut.Lines.Add(EmptyStr);
end;

procedure TForm1.ChangeState(State: string; APoint: TPoint);
begin
  (*
  LBProcessState.Caption := Format('Process status: %s', [LowerCase(State)]);
  *)
  LBProcessState.Caption := Format('Process status: %s', [
    GetEnumName(TypeInfo(TEstadoCon), Ord(Proc1.State))
  ]);
end;

procedure TForm1.InitScreen(const AGrid: TtsGrid; iFrom, iTo: integer);
var
  i: integer;
begin
  for i := iFrom to iTo do
    MMEngineOut.Lines.Add(AGrid[i]);
end;

procedure TForm1.RefreshLine(const AGrid: TtsGrid; iFrom, HeightScr: integer);
var
  j: integer;
begin
  j := MMEngineOut.Lines.Count - HeightScr - 1;
  MMEngineOut.Lines[j + iFrom] := AGrid[iFrom];
end;

procedure TForm1.RefreshLines(const AGrid: TtsGrid; iFrom, iTo, HeightScr: integer);
var
  i, j: integer;
begin
  j := MMEngineOut.Lines.Count - HeightScr - 1;
  for i := iFrom to iTo do
    MMEngineOut.Lines[j + i] := AGrid[i];
end;

end.

